import sys

# n-ary huffman coding
# ====================

freqsEN = [
    ("a", 0.081, True),
    ("b", 0.01492, True),
    ("c", 0.02782, True),
    ("d", 0.04253, True),
    ("e", 0.12702, True),
    ("f", 0.02228, True),
    ("g", 0.02015, True),
    ("h", 0.06094, True),
    ("i", 0.06966, True),
    ("j", 0.00153, True),
    ("k", 0.00772, True),
    ("l", 0.04025, True),
    ("m", 0.02406, True),
    ("n", 0.06749, True),
    ("o", 0.07507, True),
    ("p", 0.01929, True),
    ("q", 0.00095, True),
    ("r", 0.05987, True),
    ("s", 0.06327, True),
    ("t", 0.09056, True),
    ("u", 0.02758, True),
    ("v", 0.00978, True),
    ("w", 0.02360, True),
    ("x", 0.00150, True),
    ("y", 0.01974, True),
    ("z", 0.00074, True)
]

freqsCZ = [
    ("a", 0.0958926880764859, True),
    ("c", 0.029990772478052406, True),
    ("b", 0.01776119808212871, True),
    ("e", 0.10904081228988101, True),
    ("d", 0.037748410081427354, True),
    ("g", 0.0021981153155269293, True),
    ("f", 0.0017506041438910473, True),
    ("i", 0.06686137629218998, True),
    ("h", 0.024974815089542916, True),
    ("k", 0.03528082070443306, True),
    ("j", 0.023057594020051583, True),
    ("m", 0.0360544953093655, True),
    ("l", 0.05720781858830824, True),
    ("o", 0.08029998858957019, True),
    ("n", 0.059172435666639524, True),
    ("q", 0.000059327428056217, True),
    ("p", 0.031149614684546245, True),
    ("s", 0.0558597027897785, True),
    ("r", 0.04396827451223763, True),
    ("u", 0.03579031160087163, True),
    ("t", 0.053852891052334234, True),
    ("w", 0.0005426556421720198, True),
    ("v", 0.03952463505587632, True),
    ("y", 0.028575116031877037, True),
    ("x", 0.00035909890666695985, True),
    ("z", 0.03302642756808887, True),
]

# MAIN SETTINGS ======

freqs = freqsCZ
text = "jirka balhar lorem ipsum dolor sit amet ctyricet dva test test"
# symbols = ["0", "1"]
# symbols = ["/", "\\", "-"]
symbols = ["1", "2", "3", "4", "5"]
# symbols = ["▙", "▛", "▜", "▟", "█"]
# symbols = ["⌞", "⌟", "⌝", "⌜", "⌑"]

# kontrola
fsum = 0
for n in freqs:
    fsum += n[1]
print("sum of the weights:", fsum)
# /kontrola

# fill freqs with dummy nodes
# (every inner node needs to have _n_ children - most importantly the root
# so we add dummy nodes so that n_nodes % (n_symbols - 1) == 1
# https://en.wikipedia.org/wiki/Huffman_coding#n-ary_Huffman_coding
mod_now = len(freqs) % (len(symbols) - 1)
dummy_count = 0
if mod_now == 0:
    dummy_count = 1
if mod_now > 1:
    dummy_count = len(symbols) - mod_now
print("adding", dummy_count, "dummy nodes")
for i in range(dummy_count):
    freqs.append(("#%d" % i, 0, True))


# building the Huffman coding tree
def sort_desc_by_freq(freqsList):
    return sorted(freqsList, key=lambda node: node[1], reverse=True)


freqs = sort_desc_by_freq(freqs)
while len(freqs) > 1:
    basket = []
    basketFreq = 0
    while len(freqs) > 0 and len(basket) < len(symbols):
        node = freqs.pop()
        basket.append(node)
        basketFreq += node[1]

    freqs.append((basket, basketFreq, False))
    freqs = sort_desc_by_freq(freqs)

# generate letter->code table by traversing the tree
dictionary = {}


def df_codes(node, code):
    if node[2]:
        dictionary[node[0]] = code
    else:
        for child in zip(node[0], symbols):
            df_codes(child[0], code + child[1])


# pretty print the huffman tree
def df_print(node, indent):
    if node[2]:
        sys.stdout.write(indent + "└─" + node[0] + "=" + dictionary[node[0]])
        sys.stdout.write("\n")
    else:
        for child in zip(node[0], symbols):
            this_last = 1 if child[1] == symbols[-1] else 0
            kuk1 = "└─" if this_last else "├─"
            kuk2 = "  " if this_last else "│ "
            sys.stdout.write(indent + kuk1 + child[1])
            sys.stdout.write("\n")
            df_print(child[0], indent + kuk2)


df_codes(freqs[0], "")
print("Code alphabet:")
print("==============")
for key in dictionary:
    if key[0] != "#":
        print(key, "=", dictionary[key])
print()
print("Huffman tree:")
print("==============")
df_print(freqs[0], "")
print()
print("Encoded text:")
print("==============")

code = []
for c in text:
    if c in dictionary:
        code.append(dictionary[c])
    else:
        code.append(c)
code = "".join(code)
print(code)
# print in block
# cc = 0
# for c in code:
#     sys.stdout.write(c)
#     cc += 1
#     if cc % 10 == 0:
#         sys.stdout.write("|\n")

print("\n")
print("text length:", len(text))
print("code length:", len(code))

# visual output
# note: works for 5 symbols only
from PIL import Image, ImageDraw

if len(symbols) != 5:
    exit(0)

img = Image.new('RGB', (100, 100), 0xFFFFFF)
d = ImageDraw.Draw(img)

x = 0
y = 0
s = 10
for c in code:
    fill = (0, 0, 0)
    tri = []
    if c == symbols[0]:
        tri = [(x * s, y * s), (x * s + s, y * s + s), (x * s, y * s + s)]
    if c == symbols[1]:
        tri = [(x * s, y * s), (x * s + s, y * s + s), (x * s + s, y * s)]
    if c == symbols[2]:
        tri = [(x * s, y * s + s), (x * s + s, y * s), (x * s, y * s)]
    if c == symbols[3]:
        tri = [(x * s, y * s + s), (x * s + s, y * s), (x * s + s, y * s + s)]
    if c == symbols[4]:
        tri = [(x * s, y * s), (x * s + s, y * s), (x * s + s, y * s + s), (x * s, y * s + s)]

    if tri:
        d.polygon(tri, fill=fill)
    x += 1
    if x * s >= img.width:
        x = 0
        y += 1

# show the output
img.show()
